@@ -5,18 +5,18 @@ This module is unsupported and is primairily used in the PyObjC
 testsuite.
 """
 from __future__ import print_function
-import plistlib as _pl
-import unittest as _unittest
-import objc
-import os as _os
-import gc as _gc
-import subprocess as _subprocess
-import sys as _sys
-import struct as _struct
-from distutils.sysconfig import get_config_var as _get_config_var
-import re as _re
-import warnings
+
 import contextlib
+import gc as _gc
+import os as _os
+import plistlib as _pl
+import re as _re
+import struct as _struct
+import sys as _sys
+import unittest as _unittest
+from distutils.sysconfig import get_config_var as _get_config_var
+
+import objc
 
 # Ensure that methods in this module get filtered in the tracebacks
 # from unittest
@@ -25,7 +25,7 @@ __unittest = False
 # Have a way to disable the autorelease pool behaviour
 _usepool = not _os.environ.get("PYOBJC_NO_AUTORELEASE")
 
-# Python 2/3 Compatibility for the PyObjC Test Suite
+# XXX: Python 2 Compatibility for the PyObjC Test Suite
 try:
     unicode
 except NameError:
@@ -41,18 +41,13 @@ try:
 except NameError:
     basestring = str
 
-try:
-    bytes
-except NameError:
-    bytes = str
-
 try:
     unichr
 except NameError:
     unichr = chr
 
 
-def _typemap(tp):  # XXX: Is this needed?
+def _typemap(tp):
     if tp is None:
         return None
     return (
@@ -77,7 +72,7 @@ def pyobjc_options(**kwds):
             setattr(objc.options, k, orig[k])
 
 
-def sdkForPython(_cache=[]):
+def sdkForPython(_cache=[]):  # noqa: B006, M511
     """
     Return the SDK version used to compile Python itself,
     or None if no framework was used
@@ -418,6 +413,282 @@ class TestCase(_unittest.TestCase):
     This also adds a number of useful assertion methods
     """
 
+    # New API for testing function/method signatures, with one assert for
+    # the callable and one assert each for every return value and argument.
+    #
+    # Primary reason for the new API is to ensure that all metadata overrides
+    # are explicitly tested.
+
+    def assertManualBinding(self, func):
+        if hasattr(func, "__metadata__"):
+            self.fail(f"{func} has automatic bindings")
+
+    def assertCallable(self, func, *, argcount, variadic=None, deprecated=None):
+        """
+        Assert basic information about a method or function
+        """
+
+        results = []
+        if not hasattr(func, "__metadata__"):
+            self.fail("{func} has manual bindings")
+
+        info = func.__metadata__()
+
+        if isinstance(func, objc.selector):
+            cnt = info["arguments"] - 2
+        else:
+            cnt = info["arguments"]
+
+        if cnt != argcount:
+            results.append(f"{cnt} argument, expected {argcount}")
+
+        if variadic is not None:
+            if func.get("variadic") is not variadic:
+                results.append(f"variadic {info['variadic']!r} is not {variadic!r}")
+
+        else:
+            if info.get("variadic") is True:
+                results.append("variadic is True, not tested")
+
+        if deprecated is not None:
+            if info.get("deprecated") != deprecated:
+                results.append(f"variadic {info['deprecated']!r} != {deprecated!r}")
+
+        else:
+            if info.get("deprecated") is not None:
+                results.append("deprecated is set, not tested")
+
+        if results:
+            self.fail(f"{func}: " + ", ".join(results))
+
+    def assertReturns(
+        self,
+        func,
+        *,
+        type_encoding=None,
+        null_accepted=None,
+        already_retained=None,
+        already_cfretained=None,
+        callable=None,  # noqa: A002
+        c_array_of_variable_length=None,
+        c_array_delimited_by_null=None,
+    ):
+        """
+        Assert information about a return value
+        """
+
+        results = []
+        info = func.__metadata__()["retval"]
+
+        if type_encoding is not None:
+            if info["type"] != type_encoding:
+                results.append(f"encoding {info['type']!r} != {type_encoding!r}")
+
+        if null_accepted is not None:
+            if info["null_accepted"] is not null_accepted:
+                results.append(
+                    f"null_accepted {info['null_accepted']!r} is not {null_accepted!r}"
+                )
+
+        else:
+            if info["null_accepted"] is False:
+                results.append("null_accepted is False, not tested")
+
+        if already_retained is not None:
+            if info["already_retained"] is not already_retained:
+                results.append(
+                    f"already_retained {info['already_retained']!r} is not {already_retained!r}"
+                )
+
+        else:
+            if info["already_retained"] is True:
+                results.append("already_retained is True, not tested")
+
+        if already_cfretained is not None:
+            if info["already_cfretained"] is not already_cfretained:
+                results.append(
+                    f"already_cfretained {info['already_cfretained']!r} is not {already_cfretained!r}"
+                )
+
+        else:
+            if info["already_cfretained"] is True:
+                results.append("already_cfretained is True, not tested")
+
+        if c_array_of_variable_length is not None:
+            if info["c_array_of_variable_length"] is not c_array_of_variable_length:
+                results.append(
+                    f"c_array_of_variable_length {info['c_array_of_variable_length']!r} is not {c_array_of_variable_length!r}"
+                )
+
+        else:
+            if info["c_array_of_variable_length"] is True:
+                results.append("c_array_of_variable_length is True, not tested")
+
+        if c_array_delimited_by_null is not None:
+            if info["c_array_delimited_by_null"] is not c_array_delimited_by_null:
+                results.append(
+                    f"c_array_delimited_by_null {info['c_array_delimited_by_null']!r} is not {c_array_delimited_by_null!r}"
+                )
+
+        else:
+            if info["c_array_delimited_by_null"] is True:
+                results.append("c_array_delimited_by_null is True, not tested")
+
+        if callable is not None:
+            raise RuntimeError("Need to design API for this")
+
+        if results:
+            self.fail(f"{func} return-value: " + ", ".join(results))
+
+    def assertArgument(
+        self,
+        func,
+        argno,
+        *,
+        type_encoding=None,
+        type_modifier=None,
+        null_accepted=None,
+        printf_format=None,
+        already_retained=None,
+        already_cfretained=None,
+        callable=None,  # noqa: A002
+        callable_retaind=None,
+        c_array_length_in_arg=None,
+        c_array_length_in_result=None,
+        c_array_of_variable_length=None,
+        c_array_delimited_by_null=None,
+    ):
+        """
+        Assert information about an argument
+        """
+        try:
+            if isinstance(func, objc.selector):
+                info = func.__metadata__()["arguments"][argno + 2]
+            else:
+                info = func.__metadata__()["arguments"][argno]
+        except IndexError:
+            self.fail(f"{func.__name__!r} does not have argument {argno}")
+
+        results = []
+        have_modifier = False
+        have_array = False
+
+        if type_encoding is not None:
+            if type_modifier is not None:
+                expected_type = type_modifier + type_encoding
+
+            else:
+                expected_type = type_encoding
+
+            if info["type"] != expected_type:
+                results.append(f"encoding {info['type']!r} != {expected_type!r}")
+
+        elif type_modifier is not None:
+            if not info["type"].startswith(type_modifier + b"^") and not info[
+                "type"
+            ].startswith(type_modifier + b"*"):
+                results.append(
+                    f"modifier {type_modifier!r} not set or set for non-pointer {info['type']!r}"
+                )
+            else:
+                have_modifier = True
+
+        else:
+            if info["type"][0] in (objc._C_IN, objc._C_OUT, objc._C_INOUT):
+                results.append("modifier is set, not tested")
+                have_modifier = True
+
+        if null_accepted is not None:
+            if info["null_accepted"] is not null_accepted:
+                results.append(
+                    f"null_accepted {info['null_accepted']!r} is not {null_accepted!r}"
+                )
+
+        else:
+            if info["null_accepted"] is False:
+                results.append("null_accepted is False, not tested")
+
+        if printf_format is not None:
+            if info["printf_format"] is not null_accepted:
+                results.append(
+                    f"printf_format {info['printf_format']!r} is not {printf_format!r}"
+                )
+
+        else:
+            if info["printf_format"] is True:
+                results.append("printf_format is True, not tested")
+
+        if already_retained is not None:
+            if info["already_retained"] is not already_retained:
+                results.append(
+                    f"already_retained {info['already_retained']!r} is not {already_retained!r}"
+                )
+
+        else:
+            if info["already_retained"] is True:
+                results.append("already_retained is True, not tested")
+
+        if already_cfretained is not None:
+            if info["already_cfretained"] is not already_cfretained:
+                results.append(
+                    f"already_cfretained {info['already_cfretained']!r} is not {already_cfretained!r}"
+                )
+
+        else:
+            if info["already_cfretained"] is True:
+                results.append("already_cfretained is True, not tested")
+
+        if c_array_of_variable_length is not None:
+            have_array = True
+            if info["c_array_of_variable_length"] is not c_array_of_variable_length:
+                results.append(
+                    f"c_array_of_variable_length {info['c_array_of_variable_length']!r} is not {c_array_of_variable_length!r}"
+                )
+
+        if c_array_length_in_arg is not None:
+            have_array = True
+            if info["c_array_length_in_arg"] != c_array_length_in_arg:
+                results.append(
+                    f"c_array_length_in_arg {info['c_array_length_in_arg']!r} != {c_array_length_in_arg!r}"
+                )
+
+        else:
+            if info["c_array_length_in_arg"] is not None:
+                results.append("c_array_length_in_arg is set, not tested")
+
+        if c_array_length_in_result is not None:
+            have_array = True
+            if info["c_array_length_in_result"] is not c_array_length_in_result:
+                results.append(
+                    f"c_array_length_in_result {info['c_array_length_in_result']!r} is not {c_array_length_in_result!r}"
+                )
+
+        else:
+            if info["c_array_length_in_result"] is True:
+                results.append("c_array_length_in_arg is True, not tested")
+
+        if c_array_delimited_by_null is not None:
+            have_array = True
+            if info["c_array_delimited_by_null"] is not c_array_delimited_by_null:
+                results.append(
+                    f"c_array_delimited_by_null {info['c_array_delimited_by_null']!r} is not {c_array_delimited_by_null!r}"
+                )
+
+        else:
+            if info["c_array_delimited_by_null"] is True:
+                results.append("c_array_delimited_by_null is True, not tested")
+
+        if have_array and not have_modifier:
+            results.append("array, not no modifier specified")
+
+        if callable is not None:
+            raise RuntimeError("Need to design API for this")
+
+        if results:
+            self.fail(f"{func} arg {argno}: " + ", ".join(results))
+
+    # ----
+
     def assertIsCFType(self, tp, message=None):
         if not isinstance(tp, objc.objc_class):
             self.fail(message or "%r is not a CFTypeRef type" % (tp,))
@@ -434,7 +705,7 @@ class TestCase(_unittest.TestCase):
         # NOTE: Don't test if this is a subclass of one of the known
         #       CF roots, this tests is mostly used to ensure that the
         #       type is distinct from one of those roots.
-        #  XXX: With the next two lines enabled there are spurious test
+        # NOTE: With the next two lines enabled there are spurious test
         #       failures when a CF type is toll-free bridged to an
         #       (undocumented) Cocoa class. It might be worthwhile to
         #       look for these, but not in the test suite.
@@ -460,8 +731,8 @@ class TestCase(_unittest.TestCase):
         if not info.get("c_array_delimited_by_null") or not info.get("variadic"):
             self.fail(
                 message
-                or "%s is not a variadic function with a null-terminated list of arguments"
-                % (method,)
+                or "%s is not a variadic function with a "
+                "null-terminated list of arguments" % (method,)
             )
 
     def assertArgIsNullTerminated(self, method, argno, message=None):
@@ -493,7 +764,8 @@ class TestCase(_unittest.TestCase):
             if not info["arguments"][argno + offset].get("c_array_of_variable_length"):
                 self.fail(
                     message
-                    or "argument %d of %r is not a variable sized array" % (argno, method)
+                    or "argument %d of %r is not a variable sized array"
+                    % (argno, method)
                 )
         except (KeyError, IndexError):
             self.fail(
@@ -504,7 +776,9 @@ class TestCase(_unittest.TestCase):
     def assertResultIsVariableSize(self, method, message=None):
         info = method.__metadata__()
         if not info.get("retval", {}).get("c_array_of_variable_length", False):
-            self.fail(message or "result of %r is not a variable sized array" % (method,))
+            self.fail(
+                message or "result of %r is not a variable sized array" % (method,)
+            )
 
     def assertArgSizeInResult(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -516,7 +790,8 @@ class TestCase(_unittest.TestCase):
             if not info["arguments"][argno + offset].get("c_array_length_in_result"):
                 self.fail(
                     message
-                    or "argument %d of %r does not have size in result" % (argno, method)
+                    or "argument %d of %r does not have size in result"
+                    % (argno, method)
                 )
         except (KeyError, IndexError):
             self.fail(
@@ -618,14 +893,15 @@ class TestCase(_unittest.TestCase):
 
     def assertResultHasType(self, method, tp, message=None):
         info = method.__metadata__()
-        type = info.get("retval").get("type", b"v")
+        typestr = info.get("retval").get("type", b"v")
         if (
-            type != tp
-            and _typemap(type) != _typemap(tp)
-            and _typealias.get(type, type) != _typealias.get(tp, tp)
+            typestr != tp
+            and _typemap(typestr) != _typemap(tp)
+            and _typealias.get(typestr, typestr) != _typealias.get(tp, tp)
         ):
             self.fail(
-                message or "result of %r is not of type %r, but %r" % (method, tp, type)
+                message
+                or "result of %r is not of type %r, but %r" % (method, tp, typestr)
             )
 
     def assertArgHasType(self, method, argno, tp, message=None):
@@ -644,16 +920,17 @@ class TestCase(_unittest.TestCase):
             )
 
         else:
-            type = i.get("type", b"@")
+            typestr = i.get("type", b"@")
 
         if (
-            type != tp
-            and _typemap(type) != _typemap(tp)
-            and _typealias.get(type, type) != _typealias.get(tp, tp)
+            typestr != tp
+            and _typemap(typestr) != _typemap(tp)
+            and _typealias.get(typestr, typestr) != _typealias.get(tp, tp)
         ):
             self.fail(
                 message
-                or "arg %d of %s is not of type %r, but %r" % (argno, method, tp, type)
+                or "arg %d of %s is not of type %r, but %r"
+                % (argno, method, tp, typestr)
             )
 
     def assertArgIsFunction(self, method, argno, sel_type, retained, message=None):
@@ -672,9 +949,9 @@ class TestCase(_unittest.TestCase):
             )
 
         else:
-            type = i.get("type", b"@")
+            typestr = i.get("type", b"@")
 
-        if type != b"^?":
+        if typestr != b"^?":
             self.fail(
                 message
                 or "arg %d of %s is not of type function_pointer" % (argno, method)
@@ -724,9 +1001,9 @@ class TestCase(_unittest.TestCase):
             )
 
         else:
-            type = i.get("type", b"@")
+            typestr = i.get("type", b"@")
 
-        if type != b"^?":
+        if typestr != b"^?":
             self.fail(
                 message or "result of %s is not of type function_pointer" % (method,)
             )
@@ -762,13 +1039,14 @@ class TestCase(_unittest.TestCase):
             offset = 0
         info = method.__metadata__()
         try:
-            type = info["arguments"][argno + offset]["type"]
+            typestr = info["arguments"][argno + offset]["type"]
         except (IndexError, KeyError):
             self.fail("arg %d of %s does not exist" % (argno, method))
 
-        if type != b"@?":
+        if typestr != b"@?":
             self.fail(
-                message or "arg %d of %s is not of type block: %s" % (argno, method, type)
+                message
+                or "arg %d of %s is not of type block: %s" % (argno, method, typestr)
             )
 
         st = info["arguments"][argno + offset].get("callable")
@@ -783,7 +1061,7 @@ class TestCase(_unittest.TestCase):
             if st["arguments"][0]["type"] != b"^v":
                 self.fail(
                     message
-                    or "arg %d of %s has an invalid block signature %r"
+                    or "arg %d of %s has an invalid block signature %r for argument 0"
                     % (argno, method, st["arguments"][0]["type"])
                 )
             for a in st["arguments"][1:]:
@@ -806,13 +1084,16 @@ class TestCase(_unittest.TestCase):
         info = method.__metadata__()
 
         try:
-            type = info["retval"]["type"]
-            if type != b"@?":
+            typestr = info["retval"]["type"]
+            if typestr != b"@?":
                 self.fail(
-                    message or "result of %s is not of type block: %s" % (method, type)
+                    message
+                    or "result of %s is not of type block: %s" % (method, typestr)
                 )
         except KeyError:
-            self.fail(message or "result of %s is not of type block: %s" % (method, b"v"))
+            self.fail(
+                message or "result of %s is not of type block: %s" % (method, b"v")
+            )
 
         st = info["retval"].get("callable")
         if st is None:
@@ -826,7 +1107,7 @@ class TestCase(_unittest.TestCase):
             if st["arguments"][0]["type"] != b"^v":
                 self.fail(
                     message
-                    or "result %s has an invalid block signature %r"
+                    or "result %s has an invalid block signature %r for argument 0"
                     % (method, st["arguments"][0]["type"])
                 )
             for a in st["arguments"][1:]:
@@ -845,6 +1126,27 @@ class TestCase(_unittest.TestCase):
                 % (method, sel_type, iface)
             )
 
+    def assertResultIsSEL(self, method, sel_type, message=None):
+        info = method.__metadata__()
+        try:
+            i = info["retval"]
+        except (KeyError, IndexError):
+            self.fail(
+                message or "result of %s has no metadata (or doesn't exist)" % (method,)
+            )
+
+        typestr = i.get("type", b"@")
+        if typestr != objc._C_SEL:
+            self.fail(message or "result of %s is not of type SEL" % (method,))
+
+        st = i.get("sel_of_type")
+        if st != sel_type and _typemap(st) != _typemap(sel_type):
+            self.fail(
+                message
+                or "result of %s doesn't have sel_type %r but %r"
+                % (method, sel_type, st)
+            )
+
     def assertArgIsSEL(self, method, argno, sel_type, message=None):
         if isinstance(method, objc.selector):
             offset = 2
@@ -859,8 +1161,8 @@ class TestCase(_unittest.TestCase):
                 or "arg %d of %s has no metadata (or doesn't exist)" % (argno, method)
             )
 
-        type = i.get("type", b"@")
-        if type != objc._C_SEL:
+        typestr = i.get("type", b"@")
+        if typestr != objc._C_SEL:
             self.fail(message or "arg %d of %s is not of type SEL" % (argno, method))
 
         st = i.get("sel_of_type")
@@ -873,10 +1175,11 @@ class TestCase(_unittest.TestCase):
 
     def assertResultIsBOOL(self, method, message=None):
         info = method.__metadata__()
-        type = info["retval"]["type"]
-        if type != objc._C_NSBOOL:
+        typestr = info["retval"]["type"]
+        if typestr != objc._C_NSBOOL:
             self.fail(
-                message or "result of %s is not of type BOOL, but %r" % (method, type)
+                message
+                or "result of %s is not of type BOOL, but %r" % (method, typestr)
             )
 
     def assertArgIsBOOL(self, method, argno, message=None):
@@ -885,11 +1188,11 @@ class TestCase(_unittest.TestCase):
         else:
             offset = 0
         info = method.__metadata__()
-        type = info["arguments"][argno + offset]["type"]
-        if type != objc._C_NSBOOL:
+        typestr = info["arguments"][argno + offset]["type"]
+        if typestr != objc._C_NSBOOL:
             self.fail(
                 message
-                or "arg %d of %s is not of type BOOL, but %r" % (argno, method, type)
+                or "arg %d of %s is not of type BOOL, but %r" % (argno, method, typestr)
             )
 
     def assertArgIsFixedSize(self, method, argno, count, message=None):
@@ -923,7 +1226,8 @@ class TestCase(_unittest.TestCase):
                 )
         except (KeyError, IndexError):
             self.fail(
-                message or "result of %s is not a C-array of length %d" % (method, count)
+                message
+                or "result of %s is not a C-array of length %d" % (method, count)
             )
 
     def assertArgSizeInArg(self, method, argno, count, message=None):
@@ -962,7 +1266,8 @@ class TestCase(_unittest.TestCase):
         if cnt != count + offset:
             self.fail(
                 message
-                or "result %s is not a C-array of with length in arg %d" % (method, count)
+                or "result %s is not a C-array of with length in arg %d"
+                % (method, count)
             )
 
     def assertArgIsOut(self, method, argno, message=None):
@@ -971,8 +1276,8 @@ class TestCase(_unittest.TestCase):
         else:
             offset = 0
         info = method.__metadata__()
-        type = info["arguments"][argno + offset]["type"]
-        if not type.startswith(b"o^") and not type.startswith(b"o*"):
+        typestr = info["arguments"][argno + offset]["type"]
+        if not typestr.startswith(b"o^") and not typestr.startswith(b"o*"):
             self.fail(
                 message or "arg %d of %s is not an 'out' argument" % (argno, method)
             )
@@ -983,8 +1288,8 @@ class TestCase(_unittest.TestCase):
         else:
             offset = 0
         info = method.__metadata__()
-        type = info["arguments"][argno + offset]["type"]
-        if not type.startswith(b"N^") and not type.startswith(b"N*"):
+        typestr = info["arguments"][argno + offset]["type"]
+        if not typestr.startswith(b"N^") and not typestr.startswith(b"N*"):
             self.fail(
                 message or "arg %d of %s is not an 'inout' argument" % (argno, method)
             )
@@ -995,9 +1300,11 @@ class TestCase(_unittest.TestCase):
         else:
             offset = 0
         info = method.__metadata__()
-        type = info["arguments"][argno + offset]["type"]
-        if not type.startswith(b"n^") and not type.startswith(b"n*"):
-            self.fail(message or "arg %d of %s is not an 'in' argument" % (argno, method))
+        typestr = info["arguments"][argno + offset]["type"]
+        if not typestr.startswith(b"n^") and not typestr.startswith(b"n*"):
+            self.fail(
+                message or "arg %d of %s is not an 'in' argument" % (argno, method)
+            )
 
     #
     # Addition assert methods, all of them should only be necessary for
@@ -1086,13 +1393,7 @@ class TestCase(_unittest.TestCase):
 
         def assertIsNotNone(self, value, message=None):
             if value is None:
-                sel.fail(message, "%r is not %r" % (value, test))
-
-    if not hasattr(_unittest.TestCase, "assertStartsWith"):  # pragma: no cover
-
-        def assertStartswith(self, value, check, message=None):
-            if not value.startswith(check):
-                self.fail(message or "not %r.startswith(%r)" % (value, check))
+                self.fail(message, "%r is not None" % (value,))
 
     if not hasattr(_unittest.TestCase, "assertHasAttr"):  # pragma: no cover
 
@@ -1120,7 +1421,8 @@ class TestCase(_unittest.TestCase):
             if not isinstance(value, types):
                 self.fail(
                     message
-                    or "%s is not an instance of %r but %s" % (value, types, type(value))
+                    or "%s is not an instance of %r but %s"
+                    % (value, types, type(value))
                 )
 
     if not hasattr(_unittest.TestCase, "assertIsNotInstance"):  # pragma: no cover
@@ -1208,7 +1510,7 @@ else:  # pragma: no cover (py2.6)
 
             self.fail("test unexpectedly passed")
 
-        test.__name__ == func.__name__
+        test.__name__ = func.__name__
 
         return test
 
