@@ -3,18 +3,13 @@ This module implements a callback function that is used by the C code to
 add Python special methods to Objective-C classes with a suitable interface.
 """
 from objc._objc import (
-    selector,
-    lookUpClass,
-    currentBundle,
-    repythonify,
-    splitSignature,
     _block_call,
+    _rescanClass,
+    currentBundle,
+    lookUpClass,
     options,
+    selector,
 )
-from objc._objc import registerMetaDataForSelector, _updatingMetadata, _rescanClass
-import sys
-import warnings
-import collections
 
 __all__ = ("addConvenienceForClass", "registerABCForClass")
 
@@ -26,7 +21,6 @@ def register(f):
     options._class_extender = f
 
 
-# XXX: interface is too wide (super_class is not needed, can pass actual class)
 @register
 def add_convenience_methods(cls, type_dict):
     """
@@ -68,6 +62,7 @@ def registerABCForClass(classname, *abc_class):
     Register *classname* with the *abc_class*-es when
     the class becomes available.
     """
+    global CLASS_ABC
     try:
         CLASS_ABC += tuple(abc_class)
     except KeyError:
