@@ -1,11 +1,12 @@
-from PyObjCTools.TestSupport import *
 import plistlib
 
+from PyObjCTools.TestSupport import TestCase
+import objc
+
 
 class TestVersionSupport(TestCase):
     def test_macos_available(self):
         self.assertFalse(objc.macos_available(11, 20, 20))
-        self.assertFalse(objc.macos_available(10, 99, 0))
 
         with open("/System/Library/CoreServices/SystemVersion.plist", "rb") as fp:
             if hasattr(plistlib, "load"):
@@ -27,8 +28,6 @@ class TestVersionSupport(TestCase):
         self.assertTrue(objc.macos_available(version[0], version[1] - 1))
 
         if version[2]:
-            self.assertTrue(objc.macos_available(version[0], version[1], version[2] - 1))
-
-
-if __name__ == "__main__":
-    main()
+            self.assertTrue(
+                objc.macos_available(version[0], version[1], version[2] - 1)
+            )
